"use strict";

this.name = "VimanaX_Alerts";
this.version = "1.0";
this.author = "Gnievmir";
this.license = "CC-BY-NC-SA 3.0";

this._alertTimer = null;		//store pointer to graphical info timer
this._lastAlert = [];
this._incomingMiss = false;		//incoming missiles flag
this._disabled = false;

this._alerts = [
    ["vx_clean.png", "", "", ""],                                   //0 -no alert
    ["vx_d_missile.png", "DANGER!", "INCOMING MISSILE", ""],        //1 
    ["vx_d_altitude.png", "DANGER!", "ALTITUDE", "TOO LOW"],        //2
    ["vx_d_temp.png", "DANGER!", "TEMPERATURE", "CRITICAL LEVEL"],  //3
    ["vx_d_energy.png", "DANGER!", "ENERGY", "CRITICAL LEVEL"],     //4 
    ["vx_d_shield.png", "DANGER!", "SHIELD", "CRITICAL LEVEL"],     //5 
    ["vx_w_energy.png", "WARNING!", "ENERGY", "LOW LEVEL"],         //6
    ["vx_w_shield.png", "WARNING!", "SHIELD", "LOW LEVEL"],         //7
    ["vx_w_temp.png", "WARNING!", "TEMPERATURE", "TOO HIGH"]        //8 
];

this._operationalFunctionNames = [ // list of functions to enable/disable at HUDSelector callback
    "shipAttackedWithMissile",
    "shipWillLaunchFromStation",
    "shipWillDockWithStation",
    "shipDied"
];

//----------------------------------------------------------------------------------------
this.shipAttackedWithMissile = function (missile, whom) {
    this._incomingMiss = true;
    if (this._lastAlert[0] != 1) this._lastAlert[0] = -1;
    this.$display(1);
}

//----------------------------------------------------------------------------------------
this.shipWillLaunchFromStation = function () {
    if (!this._alertTimer) this._alertTimer = new Timer(this, this.$alerts.bind(this), 0, 0.5);
    this.$resetAlerts();
}

//----------------------------------------------------------------------------------------
this.shipWillDockWithStation = function () {
    var p = player.ship;
    if (this._alertTimer) {
        this._alertTimer.stop();
        delete this._alertTimer;
    }
    p.setCustomHUDDial("VimanaX_Alert", this._alerts[0][0]);
    p.setCustomHUDDial("VimanaX_AlertText1_Red", "");
    p.setCustomHUDDial("VimanaX_AlertText2_Red", "");
    p.setCustomHUDDial("VimanaX_AlertText3_Red", "");
    p.setCustomHUDDial("VimanaX_AlertText1_Orange", "");
    p.setCustomHUDDial("VimanaX_AlertText2_Orange", "");
    p.setCustomHUDDial("VimanaX_AlertText3_Orange", "");
}

//----------------------------------------------------------------------------------------
this.shipDied = function (whom, why) {
    this.shipWillDockWithStation();
}

//----------------------------------------------------------------------------------------
this.$resetAlerts = function () {
    for (let i = 1; i < 5; i++) {
        this._lastAlert[i] = 100000;
    }
    this._lastAlert[5] = 0;
}

//----------------------------------------------------------------------------------------
this.$alerts = function _vimanaXHUD_alerts() {
    var p = player.ship;
    if (!p || !p.isValid) return;
    var ano = 0;
    //check incoming missile
    if (this._incomingMiss) {
        if (player && p) {
            var miss = system.entitiesWithScanClass("CLASS_MISSILE", p, 25600);
            for (let i = 0; i < miss.length; i++) {
                if (miss[i].target == p) {
                    this.$display(1);
                    this._incomingMiss = true;
                    return;
                } else { 	// No incoming missiles
                    this._incomingMiss = false;
                }
            }
        }
    }
    //calculate altitude
    var ss = system.sun;
    var alt = 0;
    var sp = system.planets;
    var len = sp.length;
    var ctmp = Math.round(100 * p.temperature);

    if (!system.isInterstellarSpace) {
        if (ss) var d = p.position.distanceTo(ss.position);
        else var d = 100000000000000000000.0;

        for (let i = 0; i < len; i++) {
            var s = p.position.distanceTo(sp[i].position) - sp[i].radius;
            if (d > s) {
                d = s;
                ss = sp[i];
            }
        }
        if (ss) alt = p.position.distanceTo(ss) - ss.radius - p.collisionRadius;
    } else {
        alt = 999999;
    }

    if (p.energy < p.maxEnergy / 3 && p.energy < this._lastAlert[1]) {
        ano = 4;
        this._lastAlert[1] = p.energy;
    } else if (p.forwardShield < p.maxForwardShield * 0.2 && p.forwardShield < this._lastAlert[2]) {
        ano = 5;
        this._lastAlert[2] = p.forwardShield;
    } else if (p.aftShield < p.maxAftShield * 0.2 && p.aftShield < this._lastAlert[3]) {
        ano = 5;
        this._lastAlert[3] = p.aftShield;
    } else if (alt < 10000 && alt < this._lastAlert[4]) {
        ano = 2;
        this._lastAlert[4] = alt;
    } else if (ctmp > 70 && ctmp > this._lastAlert[5]) {
        ano = 3;
        this._lastAlert[5] = ctmp;
    } else if (p.energy < p.maxEnergy / 2 && p.energy < this._lastAlert[1]) {
        ano = 6;
        this._lastAlert[1] = p.energy;
    } else if (!p.torusEngaged && p.forwardShield < p.maxForwardShield / 2 && p.forwardShield < this._lastAlert[2]) {
        ano = 7;
        this._lastAlert[2] = p.forwardShield;
    } else if (!p.torusEngaged && p.aftShield < p.maxAftShield / 2 && p.aftShield < this._lastAlert[3]) {
        ano = 7;
        this._lastAlert[3] = p.aftShield;
    } else if (ctmp > 50 && ctmp > this._lastAlert[5]) {
        ano = 8;
        this._lastAlert[5] = ctmp;
    }
    this.$display(ano);
}

//----------------------------------------------------------------------------------------
this.$display = function (a) {
    if (a == 0 && a == this._lastAlert[0]) {
        this.$resetAlerts();
        return;
    }
    var p = player.ship;
    if (a > 0) { //alerts
        var seq = Math.floor((this._alertTimer.nextTime) % 5);
        if (seq < 4) {
            if (this._disabled == false) { // we're just disabling the display of the alert, not any of the calcs
                p.setCustomHUDDial("VimanaX_Alert", this._alerts[a][0]);
                if (this._alerts[a][0].indexOf("_d_") >= 0) {
                    p.setCustomHUDDial("VimanaX_AlertText1_Red", this._alerts[a][1]);
                    p.setCustomHUDDial("VimanaX_AlertText2_Red", this._alerts[a][2]);
                    p.setCustomHUDDial("VimanaX_AlertText3_Red", this._alerts[a][3]);
                    p.setCustomHUDDial("VimanaX_AlertText1_Orange", "");
                    p.setCustomHUDDial("VimanaX_AlertText2_Orange", "");
                    p.setCustomHUDDial("VimanaX_AlertText3_Orange", "");
                } else {
                    p.setCustomHUDDial("VimanaX_AlertText1_Red", "");
                    p.setCustomHUDDial("VimanaX_AlertText2_Red", "");
                    p.setCustomHUDDial("VimanaX_AlertText3_Red", "");
                    p.setCustomHUDDial("VimanaX_AlertText1_Orange", this._alerts[a][1]);
                    p.setCustomHUDDial("VimanaX_AlertText2_Orange", this._alerts[a][2]);
                    p.setCustomHUDDial("VimanaX_AlertText3_Orange", this._alerts[a][3]);
                }
            }
        } else {
            p.setCustomHUDDial("VimanaX_Alert", this._alerts[0][0]);
            p.setCustomHUDDial("VimanaX_AlertText1_Red", this._alerts[0][1]);
            p.setCustomHUDDial("VimanaX_AlertText2_Red", this._alerts[0][2]);
            p.setCustomHUDDial("VimanaX_AlertText3_Red", this._alerts[0][3]);
            p.setCustomHUDDial("VimanaX_AlertText1_Orange", this._alerts[0][1]);
            p.setCustomHUDDial("VimanaX_AlertText2_Orange", this._alerts[0][2]);
            p.setCustomHUDDial("VimanaX_AlertText3_Orange", this._alerts[0][3]);
        }
    } else {
        p.setCustomHUDDial("VimanaX_Alert", this._alerts[0][0]);
        p.setCustomHUDDial("VimanaX_AlertText1_Red", this._alerts[0][1]);
        p.setCustomHUDDial("VimanaX_AlertText2_Red", this._alerts[0][2]);
        p.setCustomHUDDial("VimanaX_AlertText3_Red", this._alerts[0][3]);
        p.setCustomHUDDial("VimanaX_AlertText1_Orange", this._alerts[0][1]);
        p.setCustomHUDDial("VimanaX_AlertText2_Orange", this._alerts[0][2]);
        p.setCustomHUDDial("VimanaX_AlertText3_Orange", this._alerts[0][3]);
    }
    this._lastAlert[0] = a;
}

//----------------------------------------------------------------------------------------
this.$disableAlerts = function() {
    if (this._alertTimer && this._alertTimer.isRunning) this._alertTimer.stop();
    var ws = worldScripts.VimanaX_Alerts;
    var funcNames = this._operationalFunctionNames;
    var i = funcNames.length;
    while (i--)
        if (ws[funcNames[i]]) {
            ws["$save_" + funcNames[i]] = ws[funcNames[i]];
            delete ws[funcNames[i]];
        }
}

//----------------------------------------------------------------------------------------
this.$enableAlerts = function() {
    var ws = worldScripts.VimanaX_Alerts;
    var funcNames = this._operationalFunctionNames;
    var i = funcNames.length;
    while (i--)
    if (!ws[funcNames[i]]) {
        ws[funcNames[i]] = ws["$save_" + funcNames[i]];
    }
}